@@ -2,11 +2,12 @@
 import logging
 from argparse import ArgumentParser
 from collections import deque
-from time import sleep
+from time import sleep, time
+from datetime import date, timedelta  # noqa: F401
 
 import yaml
 
-from mppsolar.libs.mqttbroker import MqttBrokerC as MqttBroker
+from mppsolar.libs.mqttbrokerc import MqttBroker
 from mppsolar.outputs import output_results
 from mppsolar.ports import get_port
 from mppsolar.protocols import get_protocol
@@ -17,8 +18,6 @@ from mppsolar.version import __version__  # noqa: F401
 
 # Set-up logger
 log = logging.getLogger("")
-FORMAT = "%(asctime)-15s:%(levelname)s:%(module)s:%(funcName)s@%(lineno)d: %(message)s"
-logging.basicConfig(format=FORMAT)
 
 
 class ConfigError(Exception):
@@ -35,6 +34,7 @@ sample_config = """
     - command: QPI
       outputs:
       - name: screen
+  loop: once
 """
 
 ADHOC_COMMANDS = deque([])
@@ -42,7 +42,7 @@ ADHOC_COMMANDS = deque([])
 
 
 def mqtt_callback(client, userdata, msg):
-    print(f"Received `{msg.payload}` on topic `{msg.topic}`")
+    log.info(f"Received `{msg.payload}` on topic `{msg.topic}`")
     # TODO: define message format and extract command and config
     newCommand = msg.payload
     ADHOC_COMMANDS.append(newCommand)
@@ -63,36 +63,15 @@ def readConfigFile(configFile=None):
 
 def processCommandLineOverrides(args):
     _config = {}
+    if args.once:
+        _config["loop"] = "once"
+    if args.info:
+        _config["debuglevel"] = logging.INFO
+    if args.debug:
+        _config["debuglevel"] = logging.DEBUG
     return _config
 
 
-def buildMqttBroker(config):
-    _mqttbroker = MqttBroker(config)
-    if "mqttbroker" not in config:
-        log.debug("No mqttbroker definition in config")
-        return None
-    if "name" not in config["mqttbroker"] or config["mqttbroker"]["name"] is None:
-        log.debug("No mqttbroker name defined in config")
-        return None
-    _mqttbroker = MqttBroker(
-        name=config["mqttbroker"]["name"],
-        port=config["mqttbroker"]["port"],
-        username=config["mqttbroker"]["user"],
-        password=config["mqttbroker"]["pass"],
-    )
-    # sub to command topic if defined
-    if "adhoc_commands" in config and "topic" in config["adhoc_commands"]:
-        adhoc_commands_topic = config["adhoc_commands"]["topic"]
-        if adhoc_commands_topic is not None:
-            # TODO: move to mqttbroker -
-            _mqttbroker.connect()
-            _mqttbroker.subscribe(adhoc_commands_topic, mqtt_callback)
-            # connect to mqtt
-            # TODO: move to mqttbroker -
-            _mqttbroker.start()
-    return _mqttbroker
-
-
 def main():
     description = f"Power Device Monitoring Utility, version: {__version__}"
     parser = ArgumentParser(description=description)
@@ -103,16 +82,17 @@ def main():
         nargs="?",
         type=str,
         help="Full location of config file",
-        const="/etc/mpp-solar/powermon.yml",
+        const="./powermon.yaml",
         default=None,
     )
     parser.add_argument(
         "-v", "--version", action="store_true", help="Display the version"
     )
     parser.add_argument(
-        "--generateConfigFile",
+        "-d",
+        "--dumpConfig",
         action="store_true",
-        help="Print a new config file based on options supplied (including the existing config file)",
+        help="Print the config based on options supplied (including the defaults, config file and any overrides)",
     )
     parser.add_argument(
         "-1",
@@ -129,20 +109,22 @@ def main():
     parser.add_argument(
         "-I", "--info", action="store_true", help="Enable Info and above level messages"
     )
+    parser.add_argument(
+        "--adhoc",
+        type=str,
+        default=None,
+        help="Send adhoc command to mqtt adhoc command queue - needs config file specified and populated",
+    )
 
     args = parser.parse_args()
     # prog_name = parser.prog
 
-    # logging (DEBUG, INFO, WARNING, ERROR, CRITICAL)
-    # Turn on debug if needed
+    # Temporarily set debug level based on command line options
+    log.setLevel(logging.WARNING)
+    if args.info:
+        log.setLevel(logging.INFO)
     if args.debug:
         log.setLevel(logging.DEBUG)
-    elif args.info:
-        log.setLevel(logging.INFO)
-    else:
-        # set default log level
-        log.setLevel(logging.WARNING)
-    logging.basicConfig()
 
     # Display version if asked
     log.info(description)
@@ -151,7 +133,7 @@ def main():
         return None
 
     # Build configuration from defaults, config file and command line overrides
-    log.info(f"Using config file:{args.configFile}")
+    log.info(f"Using config file: {args.configFile}")
     # build config - start with defaults
     config = yaml.safe_load(sample_config)
     # build config - update with details from config file
@@ -159,19 +141,26 @@ def main():
     # build config - override with any command line arguments
     config.update(processCommandLineOverrides(args))
 
+    # logging (DEBUG, INFO, WARNING, ERROR, CRITICAL)
+    log.setLevel(config.get("debuglevel", logging.WARNING))
+
     # if generateConfigFile is true then print config out
-    if args.generateConfigFile:
+    if args.dumpConfig:
         print("# yaml config for powermon")
         print("# default location /etc/mpp-solar/powermon.yml")
         print(yaml.dump(config))
         return
 
     # debug dump config
-    log.debug(config)
+    log.info(f"config: {config}")
 
     # Build mqtt broker
     mqttconfig = config.get("mqttbroker", {})
     mqtt_broker = MqttBroker(config=mqttconfig)
+    # is this just a call to send and adhoc command to the queue?
+    if args.adhoc:
+        print("ADHOC is todo")
+        return
     # sub to command topic if defined
     mqtt_broker.setAdhocCommands(
         adhoc_commands=mqttconfig.get("adhoc_commands"), callback=mqtt_callback
@@ -180,7 +169,7 @@ def main():
 
     # get port
     portconfig = config["port"].copy()
-    log.debug("portconfig", portconfig)
+    log.debug(f"portconfig: {portconfig}")
     port = get_port(portconfig)
     # port = get_port(porttype=porttype)
     if not port:
@@ -190,16 +179,23 @@ def main():
     # get protocol handler
     protocol = get_protocol(protocol=config["protocol"])
 
-    loop = True
+    # Get loop details
+    loop = config.get("loop", "once")
+    inDelay = False
+    delayRemaining = loop
+    doLoop = True
+
     try:
         # connect to port
         port.connect()
-        while loop:
+        while doLoop:
+            # Start timer
+            start_time = time()
             # loop through command list
             for command in config["commands"]:
                 # process any adhoc commands first
-                log.debug(f"adhoc command list: {ADHOC_COMMANDS}")
                 while len(ADHOC_COMMANDS) > 0:
+                    log.debug(f"adhoc command list: {ADHOC_COMMANDS}")
                     adhoc_command = (
                         ADHOC_COMMANDS.popleft().decode()
                     )  # FIXME: decode to str #
@@ -212,27 +208,46 @@ def main():
                     # TODO sort outputs
                     output_results(
                         results=results,
-                        outputs=config["adhoc_commands"],
+                        command=config["mqttbroker"]["adhoc_commands"],
                         mqtt_broker=mqtt_broker,
+                        fullconfig=config,
                     )
                 # process 'normal' commands
-                log.info(f"Processing command: {command}")
-                results = port.process_command(
-                    command=command["command"], protocol=protocol
-                )
-                log.debug(f"results {results}")
-                # send to output processor(s)
-                output_results(
-                    results=results, outputs=command["outputs"], mqtt_broker=mqtt_broker
-                )
-                # pause
-                # pause_time = config["command_pause"]
-                # log.debug(f"Sleeping for {pause_time}secs")
-            if args.once:
-                loop = False
+                if not inDelay:
+                    log.info(f"Processing command: {command}")
+                    if "f_command" in command:
+                        _command = command["f_command"]
+                        _command = eval(_command)
+                    else:
+                        _command = command["command"]
+                    results = port.process_command(command=_command, protocol=protocol)
+                    log.debug(f"results {results}")
+                    # send to output processor(s)
+                    output_results(
+                        results=results,
+                        command=command,
+                        mqtt_broker=mqtt_broker,
+                        fullconfig=config,
+                    )
+                    # pause
+                    # pause_time = config["command_pause"]
+                    # log.debug(f"Sleeping for {pause_time}secs")
+            if loop == "once":
+                doLoop = False
             else:
-                # sleep(pause_time)
-                sleep(5)
+                # Small pause to ....
+                sleep(0.1)
+                end_time = time()
+                delayRemaining = delayRemaining - (end_time - start_time)
+                if delayRemaining > 0 and not inDelay:
+                    log.debug(
+                        f"delaying for {loop}sec, delayRemaining: {delayRemaining}"
+                    )
+                    inDelay = True
+                if delayRemaining < 0:
+                    log.debug("setting inDelay to false")
+                    inDelay = False
+                    delayRemaining = loop
     except KeyboardInterrupt:
         print("KeyboardInterrupt")
     finally:
