@@ -1,7 +1,6 @@
 import logging
 
 from .pi30 import pi30
-from .protocol_helpers import crcPI as crc
 
 log = logging.getLogger("pi30max")
 
@@ -165,26 +164,28 @@ QUERY_COMMANDS = {
         "name": "QPIGS",
         "description": "General Status Parameters inquiry",
         "type": "QUERY",
+        "response_type": "INDEXED",
         "response": [
-            ["float", "AC Input Voltage", "V"],
-            ["float", "AC Input Frequency", "Hz"],
-            ["float", "AC Output Voltage", "V"],
-            ["float", "AC Output Frequency", "Hz"],
-            ["int", "AC Output Apparent Power", "VA"],
-            ["int", "AC Output Active Power", "W"],
-            ["int", "AC Output Load", "%"],
-            ["int", "BUS Voltage", "V"],
-            ["float", "Battery Voltage", "V"],
-            ["int", "Battery Charging Current", "A"],
-            ["int", "Battery Capacity", "%"],
-            ["int", "Inverter Heat Sink Temperature", "°C"],
-            ["float", "PV1 Input Current", "A"],
-            ["float", "PV1 Input Voltage", "V"],
-            ["float", "Battery Voltage from SCC", "V"],
-            ["int", "Battery Discharge Current", "A"],
+            [1, "AC Input Voltage", "float", "V", {"icon": "power-plug"}],
+            [2, "AC Input Frequency", "float", "Hz", {"icon": "current-ac"}],
+            [3, "AC Output Voltage", "float", "V", {"icon": "power-plug"}],
+            [4, "AC Output Frequency", "float", "Hz", {"icon": "current-ac"}],
+            [5, "AC Output Apparent Power", "int", "VA", {"icon": "power-plug"}],
+            [6, "AC Output Active Power", "int", "W", {"icon": "power-plug"}],
+            [7, "AC Output Load", "int", "%", {"icon": "brightness-percent"}],
+            [8, "BUS Voltage", "int", "V", {"icon": "details"}],
+            [9, "Battery Voltage", "float", "V", {"icon": "battery-outline"}],
+            [10, "Battery Charging Current", "int", "A", {"icon": "current-dc"}],
+            [11, "Battery Capacity", "int", "%", {"icon": "battery-outline"}],
+            [12, "Inverter Heat Sink Temperature", "int", "°C", {"icon": "details"}],
+            [13, "PV1 Input Current", "float", "A", {"icon": "solar-power"}],
+            [14, "PV1 Input Voltage", "float", "V", {"icon": "solar-power"}],
+            [15, "Battery Voltage from SCC", "float", "V", {"icon": "battery-outline"}],
+            [16, "Battery Discharge Current", "int", "A", {"icon": "battery-negative"}],
             [
-                "flags",
+                17,
                 "Device Status",
+                "flags",
                 [
                     "Is SBU Priority Version Added",
                     "Is Configuration Changed",
@@ -196,25 +197,27 @@ QUERY_COMMANDS = {
                     "Is AC Charging On",
                 ],
             ],
-            ["int", "Battery Voltage Offset for Fans On", "10mV"],
-            ["int", "EEPROM Version", ""],
-            ["int", "PV1 Charging Power", "W"],
+            [18, "Battery Voltage Offset for Fans On", "int", "10mV"],
+            [19, "EEPROM Version", "int", ""],
+            [20, "PV1 Charging Power", "int", "W", {"icon": "solar-power"}],
             [
-                "flags",
+                21,
                 "Device Status2",
+                "flags",
                 ["Is Charging to Float", "Is Switched On", "Is Dustproof Installed"],
             ],
-            ["option", "Solar Feed to Grid", ["Disabled", "Enabled"]],
+            [22, "Solar Feed to Grid", "option", ["Disabled", "Enabled"]],
             [
-                "keyed",
+                23,
                 "Country",
+                "keyed",
                 {
                     "00": "India",
                     "01": "Germany",
                     "02": "South America",
                 },
             ],
-            ["int", "Solar Feed to Grid Power", "W"],
+            [24, "Solar Feed to Grid Power", "int", "W"],
         ],
         "test_responses": [
             b"(227.2 50.0 230.3 50.0 0829 0751 010 447 54.50 020 083 0054 02.7 323.6 00.00 00000 00010110 00 00 00879 010\xf1\x8c\r",
@@ -713,16 +716,6 @@ QUERY_COMMANDS = {
             b"(1 030 030 080 021 55.40 224 030 0 0234y?\r",
         ],
     },
-    "QMN": {
-        "name": "QMN",
-        "description": "Model Name Inquiry",
-        "type": "QUERY",
-        "response_type": "SEQUENTIAL",
-        "response": [["bytes.decode", "Model Name", ""]],
-        "test_responses": [
-            b"(MKS2-8000\xb2\x8d\r",
-        ],
-    },
     "QET": {
         "name": "QET",
         "description": "Total PV Generated Energy Inquiry",
@@ -968,25 +961,3 @@ class pi30max(pi30):
         log.info(
             f"Using protocol {self._protocol_id} with {len(self.COMMANDS)} commands"
         )
-
-    def check_response_valid(self, response):
-        if response is None:
-            return False, {"ERROR": ["No response", ""]}
-        if len(response) <= 3:
-            return False, {"ERROR": ["Response to short", ""]}
-
-        if type(response) is str:
-            if "(NAK" in response:
-                return False, {"ERROR": ["NAK", ""]}
-            crc_high, crc_low = crc(response[:-3])
-            if [ord(response[-3]), ord(response[-2])] != [crc_high, crc_low]:
-                return False, {"ERROR": ["Invalid response CRCs", ""]}
-        elif type(response) is bytes:
-            if b"(NAK" in response:
-                return False, {"ERROR": ["NAK", ""]}
-
-            crc_high, crc_low = crc(response[:-3])
-            if response[-3:-1] != bytes([crc_high, crc_low]):
-                return False, {"ERROR": ["Invalid response CRC", ""]}
-        log.debug("CRCs match")
-        return True, {}
